# coding=utf-8
from __future__ import absolute_import, print_function, unicode_literals

from flask import g
from markdown import Markdown
from mock import MagicMock, patch
from six import text_type
from six.moves.urllib.parse import quote_plus

from abilian.sbe.apps.wiki.markup import SBEWikiLinkExtension
from abilian.sbe.apps.wiki.models import WikiPage

pytest_plugins = ['abilian.sbe.apps.communities.tests.fixtures']


def test_wikilink_extension():
    text = '/*€('
    wikilink = '[[' + text + ']]'

    def build_url(label, base, end):
        return '/?title=' + quote_plus(label.encode('utf-8')) + end

    extension = SBEWikiLinkExtension([('build_url', build_url)])
    ctx = {}
    ctx['extensions'] = [extension, 'toc']
    ctx['output_format'] = 'html5'
    md = Markdown(**ctx)

    page_exists_mock = MagicMock(return_value=True)
    with patch('abilian.sbe.apps.wiki.forms.page_exists', page_exists_mock):
        result = md.convert(wikilink)

    qtext = text_type(quote_plus(text.encode('utf-8')))
    expected = '<p><a class="wikilink" href="/?title={href}/">{text}</a></p>'
    expected = expected.format(href=qtext, text=text)
    assert expected == result

    # make sur the class is 'wikilink new' if page exists'
    page_exists_mock = MagicMock(return_value=False)
    with patch('abilian.sbe.apps.wiki.forms.page_exists', page_exists_mock):
        result = md.convert(wikilink)

    qtext = text_type(quote_plus(text.encode('utf-8')))
    expected = '<p><a class="wikilink new" href="/?title={href}/">{text}</a></p>'
    expected = expected.format(href=qtext, text=text)
    assert expected == result


def test_new_page(user, req_ctx):
    g.user = user

    page = WikiPage(title="Some page", body_src='abc')
    assert page.title == 'Some page'
    assert page.name == 'Some page'
    assert page.body_src == 'abc'
    assert page.body_html == '<p>abc</p>'
    assert len(page.revisions) == 1

    revision = page.revisions[0]
    assert revision.number == 0
    assert revision.author == user


def test_rename_page(user, req_ctx):
    g.user = user

    page = WikiPage(title="Some page", body_src='abc')
    assert page.title == 'Some page'
    assert page.name == 'Some page'

    page.title = 'Title Renamed'
    assert page.title == 'Title Renamed'
    assert page.name == 'Title Renamed'

    page.name = 'Name'
    assert page.title == 'Name'
    assert page.name == 'Name'


def test_create_revision(user, req_ctx):
    g.user = user

    page = WikiPage('abc')
    page.create_revision("def", "page updated")

    assert len(page.revisions) == 2
    last_revision = page.revisions[1]
    assert last_revision.number == 1
    assert last_revision.author == user



from __future__ import absolute_import, print_function, unicode_literals

from abilian.core.models.subjects import User
from pytest import fixture

from ..models import READER, Community


@fixture
def community(db):
    community = Community(name="My Community")
    db.session.add(community)
    db.session.flush()
    return community


@fixture
def community1(db):
    community = Community(name="My Community")
    db.session.add(community)

    user = User(
        email='user_1@example.com',
        password='azerty',
        can_login=True,
    )
    db.session.add(user)
    community.set_membership(user, READER)
    community.test_user = user

    db.session.flush()
    return community


@fixture
def community2(db):
    community = Community(name="Another Community")
    db.session.add(community)

    user = User(
        email='user_2@example.com',
        password='azerty',
        can_login=True,
    )
    db.session.add(user)
    community.set_membership(user, READER)
    community.test_user = user

    db.session.flush()
    return community



from __future__ import absolute_import, print_function, unicode_literals

from abilian.core.util import BasePresenter
from flask_babel import lazy_gettext as _l


class CommunityPresenter(BasePresenter):
    @property
    def breadcrumbs(self):
        return [
            {
                'label': _l("Communities"),
                'path': "/communities/",
            },
            {
                'label': self._model.name,
            },
        ]



# coding=utf-8
"""Tests from test_community are currently refactored using pytest in this
module."""
from __future__ import absolute_import, division, print_function, \
    unicode_literals

import pytest
import six
import sqlalchemy as sa
from abilian.core.entities import Entity
from abilian.core.models.subjects import User
from abilian.testing.util import login
from flask_login import login_user, logout_user
from mock import mock
from pytest import fixture

from abilian.sbe.apps.documents.models import Folder

from .. import signals, views
from ..models import MEMBER, READER, Community, CommunityIdColumn, \
    community_content


@fixture
def community(db_session):
    community = Community(name="My Community")
    db_session.add(community)
    db_session.commit()
    return community


#
# Actual tests
#
def test_instanciation(db):
    community = Community(name="My Community")
    assert isinstance(community.folder, Folder)
    # assert isinstance(community.group, Group)


def test_default_view_kw():
    # test exceptions are handled if passed an object with 'community' attribute
    # and no community_id in kwargs. and ValueError is properly raised
    if six.PY2:
        dummy = type(b'Dummy', (object, ), {b'community': None})()
    else:
        dummy = type('Dummy', (object, ), {'community': None})()

    with pytest.raises(ValueError) as exc_info:
        views.default_view_kw({}, dummy, 'dummy', 1)

    if six.PY2:
        assert exc_info.value.message == 'Cannot find community_id value'
    else:
        assert exc_info.value.args == ('Cannot find community_id value', )


def test_default_url(app, community):
    url = app.default_view.url_for(community)
    assert url.endswith('/localhost/communities/my-community/')


def test_can_recreate_with_same_name(community, db):
    name = community.name

    db.session.delete(community)
    db.session.commit()

    community = Community(name=name)
    db.session.add(community)

    # if community.folder was not deleted, this will raise IntegrityError. Test
    # passes if no exceptions is raised
    db.session.commit()


def test_rename(community):
    NEW_NAME = "My new name"
    community.rename(NEW_NAME)
    assert community.name == NEW_NAME
    assert community.folder.name == NEW_NAME


def test_auto_slug(community):
    assert community.slug == 'my-community'


def test_membership(community, db):
    user = User(email="user@example.com")

    memberships = community.memberships
    assert memberships == []

    # setup signals testers with mocks.
    when_set = mock.MagicMock()
    when_set.mock_add_spec(['__name__'])  # required for signals
    signals.membership_set.connect(when_set)
    when_removed = mock.MagicMock()
    when_removed.mock_add_spec(['__name__'])
    signals.membership_removed.connect(when_removed)

    # invalid role
    with pytest.raises(ValueError):
        community.set_membership(user, 'dummy role name')

    assert not when_set.called
    assert not when_removed.called

    # simple member
    community.set_membership(user, "member")
    db.session.commit()

    memberships = community.memberships
    assert len(memberships) == 1
    assert memberships[0].user == user
    assert memberships[0].role is MEMBER

    when_set.assert_called_once_with(
        community,
        is_new=True,
        membership=memberships[0],
    )
    assert not when_removed.called
    when_set.reset_mock()

    assert community.get_role(user) is MEMBER

    assert community.get_memberships() == [memberships[0]]
    assert community.get_memberships('member') == [memberships[0]]
    assert community.get_memberships('manager') == []

    # change user role
    community.set_membership(user, "manager")
    db.session.commit()

    memberships = community.memberships
    assert len(memberships) == 1
    assert memberships[0].user == user
    assert memberships[0].role == "manager"

    assert community.get_role(user) == "manager"

    when_set.assert_called_once_with(
        community,
        is_new=False,
        membership=memberships[0],
    )
    assert not when_removed.called
    when_set.reset_mock()

    # remove user
    membership = memberships[0]
    community.remove_membership(user)
    db.session.commit()

    memberships = community.memberships
    assert memberships == []

    assert not when_set.called
    when_removed.assert_called_once_with(community, membership=membership)


def test_folder_roles(community, db, app):
    user = User(email="user@example.com")
    folder = community.folder
    community.set_membership(user, "member")
    db.session.commit()
    security = app.services['security']

    assert security.get_roles(user, folder) == ["reader"]

    # this tests a bug, where local roles whould disappear when setting
    # membership twice
    community.set_membership(user, "member")
    assert security.get_roles(user, folder) == ["reader"]


def test_community_content_decorator(community, db):
    @community_content
    class CommunityContent(Entity):
        community_id = CommunityIdColumn()
        community = sa.orm.relation(Community, foreign_keys=[community_id])

    sa.orm.configure_mappers()
    conn = db.session.connection()
    for table in sa.inspect(CommunityContent).tables:
        if not table.exists(conn):
            table.create(conn)

    cc = CommunityContent(name='my content', community=community)
    db.session.add(cc)
    db.session.flush()
    assert hasattr(cc, 'community_slug')
    assert cc.community_slug == 'my-community'
    assert cc.slug == 'my-content'

    index_to = dict(CommunityContent.__indexation_args__['index_to'])
    assert 'community_slug' in index_to


##########################################################################


def test_community_indexed(app, db):
    index_service = app.services['indexing']
    index_service.start()

    security_service = app.services['security']
    security_service.start()

    obj_types = (Community.entity_type, )

    user_no_community = User(email='no_community@example.com')
    db.session.add(user_no_community)

    community1 = Community(name="My Community")
    db.session.add(community1)

    community2 = Community(name='Other community')
    db.session.add(community2)

    user = User(email='user_1@example.com')
    db.session.add(user)
    community1.set_membership(user, READER)

    user_c2 = User(email='user_2@example.com')
    db.session.add(user_c2)
    community2.set_membership(user_c2, READER)

    db.session.commit()

    with app.test_request_context():
        with login(user_no_community):
            res = index_service.search('community', object_types=obj_types)
            assert len(res) == 0

        with login(user):
            res = index_service.search('community', object_types=obj_types)
            assert len(res) == 1
            hit = res[0]
            assert hit['object_key'] == community1.object_key

        with login(user_c2):
            res = index_service.search('community', object_types=obj_types)
            assert len(res) == 1
            hit = res[0]
            assert hit['object_key'] == community2.object_key


def test_default_view_kw_with_hit(app, db, community):
    index_service = app.services['indexing']
    index_service.start()

    security_service = app.services['security']
    security_service.start()

    user = User(email='user_1@example.com')
    db.session.add(user)
    community.set_membership(user, READER)

    obj_types = (Community.entity_type, )

    with app.test_request_context():
        with login(user):
            hit = index_service.search('community', object_types=obj_types)[0]
            kw = views.default_view_kw({}, hit, hit['object_type'], hit['id'])
            assert kw == {'community_id': community.slug}



# coding=utf-8
""""""
from __future__ import absolute_import, print_function, unicode_literals

from abilian.services.security import Admin
from abilian.testing.util import client_login
from flask import url_for
from flask_login import current_user
from pytest import mark

from ..models import Community


def test_index(app, client, community1):
    security_service = app.services['security']
    security_service.start()

    user = community1.test_user
    with client_login(client, user):
        response = client.get(url_for("communities.index"))
        print(current_user)
        assert response.status_code == 200


def test_community_home(app, client, community1, community2):
    security_service = app.services['security']
    security_service.start()

    url = app.default_view.url_for(community1)

    user1 = community1.test_user
    with client_login(client, user1):
        response = client.get(url)
        print(current_user)
        assert response.status_code == 302
        expected_url = url_for(
            "wall.index",
            community_id=community1.slug,
            _external=True,
        )
        assert response.headers['Location'] == expected_url

    user2 = community2.test_user
    with client_login(client, user2):
        response = client.get(url)
        print(current_user)
        # assert response.headers['Location'] == ""
        assert response.status_code == 403


def test_new(app, client, community1):
    security_service = app.services['security']
    security_service.start()

    user = community1.test_user

    with client_login(client, user):
        response = client.get(url_for("communities.new"))
        assert response.status_code == 403

        app.services['security'].grant_role(user, Admin)
        response = client.get(url_for("communities.new"))
        assert response.status_code == 200


@mark.skip  # TODO: fix
def test_community_settings(app, client, community1):
    security_service = app.services['security']
    security_service.start()

    url = url_for('communities.settings', community_id=community1.slug)
    user = community1.test_user

    with client_login(client, user):
        response = client.get(url)
        assert response.status_code == 403

        app.services['security'].grant_role(user, Admin)
        response = client.get(url)
        assert response.status_code == 200

        data = {
            '__action': 'edit',
            'name': 'edited community',
            'description': 'my community',
            'linked_group': '',
            'type': 'participative',
        }
        response = client.post(url, data=data)
        assert response.status_code == 302
        expected_url = 'http://localhost/communities/{}/'.format(
            community1.slug
        )
        assert response.headers['Location'] == expected_url

        community = Community.query.get(community1.id)
        assert community.name == 'edited community'


@mark.skip  # TODO: fix
def test_members(app, client, db, community1, community2):
    security_service = app.services['security']
    security_service.start()

    user1 = community1.test_user
    user2 = community2.test_user

    with client_login(client, user1):
        url = url_for(
            "communities.members",
            community_id=community1.slug,
        )
        response = client.get(url)
        assert response.status_code == 200

        # test add user
        data = {'action': 'add-user-role', 'user': user2.id}
        response = client.post(url, data=data)
        assert response.status_code in (403, 400)

        app.services['security'].grant_role(user1, Admin)

        data = {
            'action': 'add-user-role',
            'user': user2.id,
            'role': 'member',
        }
        response = client.post(url, data=data)
        assert response.status_code == 302
        assert response.headers['Location'] == 'http://localhost' + url

        membership = [m for m in community1.memberships if m.user == user2][0]
        assert membership.role == 'member'

        data['action'] = 'set-user-role'
        data['role'] = 'manager'
        response = client.post(url, data=data)
        assert response.status_code == 302
        assert response.headers['Location'] == 'http://localhost' + url

        db.session.expire(membership)
        assert membership.role == 'manager'

        # Community.query.session is not self.db.session, but web app
        # session.
        community = Community.query.get(community1.id)
        assert user2 in community.members

        # test delete
        data = {
            'action':
            'delete',
            'user':
            user2.id,
            'membership':
            [m.id for m in community1.memberships if m.user == user2][0],
        }
        response = client.post(url, data=data)
        expected_url = 'http://localhost/communities/{}/members'.format(
            community1.slug,
        )
        assert response.status_code == 302
        assert response.headers['Location'] == expected_url

        assert user2 not in community.members
